/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_ICMPCRAFTER_H__8BB8FDD9_9799_4359_90C5_B1FE2A83D847__INCLUDED_)
#define AFX_ICMPCRAFTER_H__8BB8FDD9_9799_4359_90C5_B1FE2A83D847__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "IPCrafter.h"
#include "ICMPSocket.h"

KOMODIA_NAMESPACE_START

class CICMPCrafter : public CIPCrafter,
					 public CICMPSocket
{
public:
	//Get the memory for the following ICMP types
	char* GetInformation(BOOL bReply,
						 unsigned short usIdentifier,
						 unsigned short usSequence,
						 unsigned short& usTotalSize)const;

	char* GetTimestamp(BOOL bReply,
					   unsigned short usIdentifier,
					   unsigned short usSequence,
					   unsigned long ulOriginateTimestamp,
					   unsigned long ulReceiveTimestamp,
					   unsigned long ulTransmitTimestamp,
					   unsigned short& usTotalSize)const;

	char* GetEcho(BOOL bReply,
				  unsigned short usIdentifier,
				  unsigned short usSequence,
				  unsigned long ulData,
				  unsigned short& usTotalSize)const;

	char* GetRedirect(unsigned char cType,
					  IP aGatewayAddress,
					  unsigned short& usTotalSize)const;

	char* GetQuench(unsigned short& usTotalSize)const;

	char* GetParameter(unsigned char cError,
					   unsigned short& usTotalSize)const;

	char* GetTime(unsigned char cType,
				  unsigned short& usTotalSize)const;

	char* GetUnreachable(unsigned char cType,
						 unsigned short& usTotalSize)const;

	//Set the destination address
	void SetDestinationAddress(IP aDestinationAddress);
	void SetDestinationAddress(const std::string& rDestinationAddress);

	//Do we use default checksum ?
	void SetICMPUseDefaultChecksum(BOOL bDefault);

	//The checksum we want to use
	void SetICMPChecksum(unsigned short usChecksum);

	//Ctor and Dtor
	CICMPCrafter();
	virtual ~CICMPCrafter();
protected:
	//Modify the ICMP
	virtual void FinalICMPHeader(LPICMPHeader lpHead)const;

	//Create an IP header
	virtual LPIpHeader ConstructIPHeader (unsigned char  ucProtocol,
										  unsigned short usFragmentationFlags,
										  unsigned char  ucTTL,
										  unsigned short usIdentification,
										  unsigned char  ucHeaderLength)const;

	//Last stop before sending the header
	virtual void FinalIPHeader(LPIpHeader lpHead)const;
private:
	//No copy Ctor
	CICMPCrafter(const CICMPCrafter& rCrafter);
	
	//No assigment operator
	CICMPCrafter& operator=(const CICMPCrafter& rCrafter);

	//Created the memory
	char* CreateFinal(unsigned char cICMP, 
					  unsigned char cType,
					  unsigned short& usTotalSize)const;

	//Create the memory
	char* CreateFinal(const LPICMPHeader pHeader,
					  unsigned short& usTotalSize)const;

	//Do we use default checksum
	BOOL m_bDefaultChecksum;

	//Checksum to modify
	unsigned short m_usChecksum;

	//The destination address
	IP m_aDestinationAddress;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_ICMPCRAFTER_H__8BB8FDD9_9799_4359_90C5_B1FE2A83D847__INCLUDED_)
