/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "ICMPSocketAsync.h"

#include "ErrorHandlerMacros.h"
#include "Array_ptr.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CICMPSocketAsync_Class "CICMPSocketAsync"

CICMPSocketAsync::CICMPSocketAsync(BOOL bRawSocket) : CICMPSocket(bRawSocket),
													  CAsyncSocket()
{
	try
	{
		//Set our name
		SetName(CICMPSocketAsync_Class);
	}
	ERROR_HANDLER("CICMPSocketAsync")
}

CICMPSocketAsync::~CICMPSocketAsync()
{
	try
	{
		//We need to close it here
		Close();
	}
	ERROR_HANDLER("~CICMPSocketAsync")
}

BOOL CICMPSocketAsync::SetAsync()
{
	try
	{
		//Set event to read / write / close / oob
		int iResult;
		iResult=InternalWSAAsyncSelect(WM_SOCKET_GENERAL,
									   FD_WRITE | FD_READ);

		//Is it OK?
		if (iResult)
		{
			//Set socket error
			SetLastError("SetAsync");

			//Exit
			return FALSE;
		}
		
		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("SetAsync",FALSE)
}

BOOL CICMPSocketAsync::OnSocketReceive(int iErrorCode)
{
	try
	{
		//Here we receive the data
		if (!iErrorCode)
		{
			//Buffer, can be up to 65535
			char* pBuffer;
			pBuffer=new char[65535];

			//Protect the data
			CArray_ptr<char> pProtection(pBuffer);

			//Read the data
			int iRead;
			iRead=Receive(pBuffer,65535);

			//Process result
			BOOL bResult=FALSE;

			//Only if not an error
			if (iRead!=GetErrorCode())
				bResult=ProccessICMP(pBuffer);

			//Exit
			return bResult;
		}
		else
			return FALSE;
	}
	ERROR_HANDLER_RETURN("OnSocketReceive",FALSE)
}

BOOL CICMPSocketAsync::Create()
{
	try
	{
		//Try to create as an ICMP socket
		if (!CICMPSocket::Create())
		{
			//Report it
			ReportError("Create","Failed to create ICMP socket!");

			//Exit
			return FALSE;
		}

		//Add the socket to the asynch list
		AddSocketToList();

		//Set async status
		return SetAsync();
	}
	ERROR_HANDLER_RETURN("Create",FALSE)
}

SOCKET CICMPSocketAsync::GetAsyncHandle() const
{
	return GetHandle();
}

BOOL CICMPSocketAsync::Close()
{
	try
	{
		//Quit if not ok
		if (!ValidSocket())
			return FALSE;

		//Kill the timer
		CAsyncSocket::SocketClosing();

		//Remove from socket list
		RemoveSocketFromList();

		return CICMPSocket::Close();
	}
	ERROR_HANDLER_RETURN("Close",FALSE)
}

KOMODIA_NAMESPACE_END