/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#ifndef _DNSQuery_H_
#define _DNSQuery_H_

#include "ErrorHandler.h"

#include <vector>

namespace KomodiaDNS
{

// Header of the DNS question
typedef struct _DnsQuestionHeader 
{
	std::string			sName;				// Variable
	unsigned short		usType;				// Too many fields
	unsigned short		usClass;

} DnsQuestionHeader;

typedef DnsQuestionHeader FAR * LPDnsQuestionHeader;

#define DnsQuestionHeaderLength sizeof(DnsQuestionHeader)

#define DnsQuestionCopyHeader sizeof(unsigned short)*2

//Our DNS query class
class CDNSQuery : public CErrorHandler
{
public:
	typedef enum _DnsQueryTypes
	{
		A=1,		//a host address
		NS=2,		//an authoritative name server
		MD=3,		//a mail destination (Obsolete - use MX)
		MF=4,		//a mail forwarder (Obsolete - use MX)
		CNAME=5,	//the canonical name for an alias
		SOA=6,		//marks the start of a zone of authority
		MB=7,		//a mailbox domain name (EXPERIMENTAL)
		MG=8,		//a mail group member (EXPERIMENTAL)
		MR=9,		//a mail rename domain name (EXPERIMENTAL)
		NLL=10,	//a null RR (EXPERIMENTAL)
		WKS=11,		//a well known service description
		PTR=12,		//a domain name pointer
		HINFO=13,	//host information
		MINFO=14,	//mailbox or mail list information
		MX=15,		//mail exchange
		TXT=16,		//text strings,
		ERR=17		//Internal error, not part of DNS
	} DNSQueryTypes;
public:
	//Get a query
	const DnsQuestionHeader& GetQuery(int iIndex)const;

	//Get teh query string
	std::string GetQueryString(int iIndex)const;

	//Get queries count
	int GetQueryCount()const;

	//Get the first query type (usually, for the answer parsing)
	DNSQueryTypes GetFirstQueryType()const;

	//Parse a buffer
	int Parse(const char* pBuffer,
			  int iNumber);
	
	//How many question do we have
	unsigned short GetQuestionCount()const;

	//Add a query to the DNS query
	void AddQuery(const std::string& rDomain,
				  DNSQueryTypes aType);

	//Get the serialization size
	unsigned short GetSerializationSize()const;

	//Serialize the buffer
	unsigned short SerializeQuery(char* pBuffer)const;

	//Copy operator
	CDNSQuery& operator=(const CDNSQuery& rQuery);

	//Ctor and Dtor
	CDNSQuery();
	CDNSQuery(const CDNSQuery& rQuery);
	virtual ~CDNSQuery();
private:
	//Our vector of questions
	typedef std::vector<DnsQuestionHeader> QueryVector;
private:
	//The Query array
	QueryVector m_aQueries;
};

//of namespace
}

#endif