/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "DNSTCPSocket.h"

#include "ErrorHandlerMacros.h"

#include "DNSSocket.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

namespace KomodiaDNS
{

#define CDNSTCPSocket_Class "CDNSTCPSocket"

#define BUFFER_SIZE 2000

CDNSTCPSocket::CDNSTCPSocket(CDNSSocket* pSocket) : CTCPSocketAsync(FALSE),
													m_pDNSSocket(pSocket),
													m_iBufferPosition(0),
													m_pBuffer(NULL)
{
	try
	{
		//Set our name
		SetName(CDNSTCPSocket_Class);

		//Allocate the buffer
		m_pBuffer=new char[BUFFER_SIZE];
	}
	ERROR_HANDLER("CDNSTCPSocket")
}

CDNSTCPSocket::~CDNSTCPSocket()
{
	try
	{
		//Delete the buffer
		delete [] m_pBuffer;
	}
	ERROR_HANDLER("~CDNSTCPSocket")
}

BOOL CDNSTCPSocket::OnSocketClose(int iErrorCode)
{
	try
	{
		//Indicate we are closed
		m_pDNSSocket->SocketClosed();
		
		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("OnSocketClose",TRUE)
}

BOOL CDNSTCPSocket::OnSocketReceive(int iErrorCode)
{
	try
	{
		//Check if there's no error
		if (!iErrorCode)
		{
			//Receive the data
			char* pBuffer;
			pBuffer=new char[BUFFER_SIZE];
			
			//Do we have fragmentation ?
			if (m_iBufferPosition)
				//Copy to the buffer
				memcpy(pBuffer,
					   m_pBuffer,
					   m_iBufferPosition);


			//How much data did we receive
			int iReceived=0;

			//Receive
			iReceived=Receive(pBuffer+m_iBufferPosition,
							  BUFFER_SIZE-m_iBufferPosition);

			//If every thing was OK
			if (iReceived==CSpoofBase::GetErrorCode())
			{
				//Inform of an error
				m_pDNSSocket->SocketError(GetSystemLastError());

				//Delete the data
				delete [] pBuffer;

				//No frags
				m_iBufferPosition=0;

				//Done
				return TRUE;
			}

			//Fix the frags
			iReceived+=m_iBufferPosition;
			m_iBufferPosition=0;

			//Check for frags
			int iPosition;
			iPosition=0;

			BOOL bQuit;
			bQuit=FALSE;

			//Start
			while (!bQuit)
			{
				//Get the frags
				unsigned short usFragSize;
				usFragSize=(((unsigned short)pBuffer[iPosition]) << 8) +
						   (unsigned char)pBuffer[iPosition+1]+2;

				//Do we have enough ?
				if (iPosition+usFragSize>iReceived)
					bQuit=TRUE;
				else if ((iPosition+usFragSize)==iReceived)
				{
					//Set the new size
					iPosition+=usFragSize;

					//Quit
					bQuit=TRUE;
				}
				else
					iPosition+=usFragSize;
			}

			//Do we have a frag
			if (iPosition<iReceived)
			{
				//Calculate the frag size
				m_iBufferPosition=iReceived-iPosition;

				//Copy the frag
				memcpy(m_pBuffer,
					   pBuffer+iPosition,
					   m_iBufferPosition);

				//Fix the receive size
				iReceived-=m_iBufferPosition;
			}

			//Do we have the data
			if (iReceived)
				//Inform our father
				m_pDNSSocket->SocketReceive(pBuffer,
											iReceived,
											TRUE);
		}
		else
			//Inform about the error
			m_pDNSSocket->SocketError(iErrorCode);

		//Always return true
		return TRUE;
	}
	ERROR_HANDLER_RETURN("OnSocketReceive",TRUE)
}

BOOL CDNSTCPSocket::OnSocketConnect(int iErrorCode)
{
	try
	{
		//Call father
		m_pDNSSocket->SocketConnected(iErrorCode);

		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("OnSocketConnect",FALSE)
}

//of namespace
}