/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "PingSocket.h"

#include "ErrorHandlerMacros.h"

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

#define CPingSocket_Class "CPingSocket"

CPingSocket::CPingSocket() : CICMPSocketAsync(FALSE),
							 m_bPingDone(TRUE),
							 m_bSameID(FALSE),
							 m_usLastPingID(0),
							 m_usLastPingSequence(0)
{
	try
	{
		//Set our name
		SetName(CPingSocket_Class);

		//Last ping ID
		while (!m_usLastPingID)
			m_usLastPingID=GetTickCount();
	}
	ERROR_HANDLER("CPingSocket")
}

CPingSocket::~CPingSocket()
{
}

BOOL CPingSocket::OnSocketReceive(int iErrorCode)
{
	try
	{
		//Receive the data, and check if it's our ping
		if (CICMPSocketAsync::OnSocketReceive(iErrorCode))
		{
			//Check if it is our ping
			if ((IsIDOK(GetLastICMPHeader()->sICMP.sUS.us1,
						GetLastICMPHeader()->sICMP.sUS.us2) && 
				 GetLastICMPHeader()->ucICMPType==ICMP_Echo_Reply) ||
				GetLastICMPHeader()->ucICMPType==ICMP_Time)
			{
				//Do we have a timer ?
				if (m_dwTimeout)
					//Reset the timeout
					CAsyncSocket::KillTimer();

				//Save some data
				//Time it took
				m_dwTimeElapsed=GetTickCount()-m_dwLastTick;

				//Save the TTL
				m_ucTTL=GetLastIPHeader()->ucTTL;

				//Get the data size
				m_iDataSize=GetLastDataSize();

				//Replying address
				m_aReplyingAddress=GetLastIPHeader()->ulSourceAddress;
		
				//Set ping done
				m_bPingDone=TRUE;

				//No timeout
				m_bTimeout=FALSE;

				//Ping is done
				PingDone(FALSE);

				//What is the sequence
				unsigned short usSequence;
				if (GetLastICMPHeader()->ucICMPType==ICMP_Time)
					usSequence=GetLastICMPICMPHeader()->sICMP.sUS.us1;
				else
					usSequence=GetLastICMPHeader()->sICMP.sUS.us1;

				//Ping is done with data
				PingDoneData(m_aReplyingAddress,
							 m_ucTTL,
							 m_dwTimeElapsed,
							 m_iDataSize,
							 usSequence,
							 GetLastICMPHeader()->ucICMPType==ICMP_Time);
			}
		}

		//Done
		return TRUE;
	}
	ERROR_HANDLER_RETURN("OnSocketReceive",TRUE)
}

BOOL CPingSocket::IsPingDone()const
{
	return m_bPingDone;
}

void CPingSocket::SetUseSameID(BOOL bSameID)
{
	m_bSameID=bSameID;
}

BOOL CPingSocket::Ping(const std::string& rDestinationAddress,
					   DWORD dwTimeout)
{
	try
	{
		return Ping(StringToLong(rDestinationAddress),
					dwTimeout);
	}
	ERROR_HANDLER_RETURN("Ping",FALSE)
}

BOOL CPingSocket::Ping(IP aDestinationAddress,
					   DWORD dwTimeout)
{
	try
	{
		//Set some initial data
		m_ucTTL=0;
		m_aReplyingAddress=0;
		m_bTimeout=FALSE;
		m_dwTimeElapsed=0;
		m_iDataSize=0;
		m_dwTimeout=dwTimeout;

		//What is the timeout
		if (dwTimeout)
			//Set the timeout
			SetTimeout(dwTimeout);

		//Set ping timer
		m_dwLastTick=GetTickCount();

		//Set ping done
		m_bPingDone=FALSE;

		if (!m_bSameID)
			//What is our ID
			m_usLastPingID=GetTickCount()*GetCurrentProcessId();

		//Send the ping
		return SendEcho(aDestinationAddress,
						FALSE,
						m_usLastPingID,
						m_usLastPingSequence,
						0)!=GetErrorCode();
	}
	ERROR_HANDLER_RETURN("Ping",FALSE)
}

BOOL CPingSocket::OnSocketTimeout()
{
	try
	{
		//Kill the timer
		KillTimer();

		//Set available new ping
		m_bPingDone=TRUE;

		//Timeout
		m_bTimeout=TRUE;

		//Ping is done
		PingDone(TRUE);

		//Done
		return FALSE;
	}
	ERROR_HANDLER_RETURN("OnSocketTimeout",FALSE)
}

void CPingSocket::PingDone(BOOL bTimeout)
{
}

IP CPingSocket::GetReplyingAddress()const
{
	return m_aReplyingAddress;
}

BOOL CPingSocket::IsTimeout()const
{
	return m_bTimeout;
}

DWORD CPingSocket::GetTimeElapsed()const
{
	return m_dwTimeElapsed;
}

unsigned char CPingSocket::GetTTL()const
{
	return m_ucTTL;
}

int CPingSocket::GetDataSize()const
{
	return m_iDataSize;
}

void CPingSocket::PingDoneData(IP aReplyingAddress,
							   unsigned char ucTTL,
							   DWORD dwTimeElapsed,
							   int iDataSize,
							   unsigned short usID,
							   BOOL bTTLExpired)
{
}

void CPingSocket::SetID(unsigned short usID)
{
	m_usLastPingID=usID;
}

void CPingSocket::SetSequence(unsigned short usSequence)
{
	m_usLastPingSequence=usSequence;
}

BOOL CPingSocket::IsIDOK(unsigned short usID,
						 unsigned short usSequence)const
{
	return m_usLastPingID==usID &&
		   m_usLastPingSequence==usSequence;
}

KOMODIA_NAMESPACE_END
