/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_REALTIMETRACEROUTESOCKET_H__311D7557_488E_46B7_8DE6_594A26F57306__INCLUDED_)
#define AFX_REALTIMETRACEROUTESOCKET_H__311D7557_488E_46B7_8DE6_594A26F57306__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PingSocket.h"
#include "DNSManager.h"

#include <map>

KOMODIA_NAMESPACE_START

class CGenericCriticalSection;

class CRealTimeTraceRouteSocket : public CPingSocket  
{
private:
	//Our DNS class
	class CRealTimeDNSManager : public KomodiaDNS::CDNSManager
	{
	public:
		//Ctor and Dtor
		CRealTimeDNSManager(CRealTimeTraceRouteSocket* pFather,
							const std::string strDNSServer,
							BOOL bTCP);
		virtual ~CRealTimeDNSManager();
	protected:
		//Events for user to override
		virtual void OnDNSError(int iErrorCode,
								LPVOID lpLocalIDData);
		virtual void OnDNSTimeout(const KomodiaDNS::CDNSQuery& rQuery,
								  LPVOID lpLocalIDData);
		virtual void OnDNSReceive(KomodiaDNS::CDNSAnswers* pAnswers,
								  LPVOID lpLocalIDData);
	private:
		CRealTimeTraceRouteSocket* m_pFather;
	};

	friend CRealTimeDNSManager;
public:
	//Max hops
	typedef enum _MaxHops
	{
		mhMaxHops=40
	} MaxHops;

	//Our data
	typedef struct _TraceRouteData
	{
		std::auto_ptr<CGenericCriticalSection>	pCSection;
		IP										aReplyingAddress;
		unsigned char							ucTTL;
		DWORD									dwElapsedTime;
		std::string								sReverseDNS;
		CRealTimeDNSManager*					pDNS;
		BOOL									bResolved;
		BOOL									bTarget;
		BOOL									bFinal;
		LPVOID									lpUserData;

		//assignment opretor
		_TraceRouteData& operator=(const _TraceRouteData& rData)
		{
			aReplyingAddress=rData.aReplyingAddress;
			bFinal=rData.bFinal;
			bResolved=rData.bResolved;
			bTarget=rData.bTarget;
			dwElapsedTime=rData.dwElapsedTime;
			lpUserData=rData.lpUserData;
			pDNS=rData.pDNS;
			ucTTL=rData.ucTTL;
			sReverseDNS=rData.sReverseDNS;
			
			//Done
			return *this;
		}
	} TraceRouteData;
	
	//Resolve thread data
	typedef struct _ResolveThreadData
	{
		TraceRouteData*				pData;
		CRealTimeTraceRouteSocket*	pClass;
	} ResolveThreadData;

public:
	//Fake the last request
	void FakeLastRequest();

	//Enable disable resolve
	void SetResolveStatus(BOOL bResolve);

	//Get max hops
	unsigned char GetMaxHops()const;

	//Get the data
	const TraceRouteData* GetData()const;

	//Are we initialized
	BOOL IsInitialized()const;

	//Is the traced done
	virtual BOOL IsTraceDone()const;

	//Do a trace
	virtual BOOL TraceRoute(const std::string& rDestinationAddress,
							DWORD dwTimeout);

	virtual BOOL TraceRoute(IP aDestinationAddress,
							DWORD dwTimeout);

	//ctor and dtor
	CRealTimeTraceRouteSocket(std::string sDNSServer,
							  BOOL bDNSTcp,
							  BOOL bMultithreaded=TRUE);
	virtual ~CRealTimeTraceRouteSocket();
protected:
	//Get the DNS manager
	CRealTimeDNSManager* GetDNSManager()const;

	//Data for modification
	virtual void TraceDataForModify(TraceRouteData* pData,
									BOOL bResolved);

	//Resolved DNS data
	virtual void TraceData(const TraceRouteData* pData,
						   BOOL bResolved);

	//Is the ID OK
	virtual BOOL IsIDOK(unsigned short usID,
						unsigned short usSequence)const;

	//Timeout indicator
	virtual BOOL OnSocketTimeout();

	//For user to overide trace is done
	virtual void TraceDone(BOOL bError);
private:
	//Dns asks if we are done
	void DNSDone();

	//Call the done routines
	void InvokeTraceDone();

	//Our ping data
	virtual void PingDoneData(IP aReplyingAddress,
							  unsigned char ucTTL,
							  DWORD dwTimeElapsed,
							  int iDataSize,
							  unsigned short usID,
							  BOOL bTTLExpired);

	//Our thread for DNS queries
	static void ResolveData(LPVOID lpData);

	//Adjust the number of resovers
	void AdjustResolvers(long lAmount);

	//Get the number of resovers
	unsigned long GetResolvers()const;

	//Allocate start of ID range
	static unsigned short GetPortStart();

	//Our port start
	static unsigned short m_sPortStart;

	//Current port start
	unsigned short m_usPortStart;

	//Our max hops array
	TraceRouteData m_aData[mhMaxHops+1];

	//Is our trace done ?
	BOOL m_bTraceDone;

	//Our DNS manager
	CRealTimeDNSManager* m_pManager;

	//What is the maximum hop
	unsigned char m_ucMaxHOP;

	//What is our minimum target hop
	unsigned char m_ucMinHOP;

	//Our CS
	CGenericCriticalSection* m_pCSection;

	//Our resolve CS
	CGenericCriticalSection* m_pCSectionResolve;

	//Our trace CS
	CGenericCriticalSection* m_pCSectionGeneral;

	//Our resolvers
	unsigned long m_ulResolvers;

	//Are we initialized
	BOOL m_bInitialized;

	//Address we are tracing
	IP m_aTracedAddress;

	//Resolve status
	BOOL m_bResolveIPs;

	//Are we partial done ?
	BOOL m_bPartialDone;

	//We are finishing
	BOOL m_bFinishing;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_REALTIMETRACEROUTESOCKET_H__311D7557_488E_46B7_8DE6_594A26F57306__INCLUDED_)
