/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TCPPORTSCANNER_H__9827BC61_F74C_408D_8D12_A99E0FBC4A9F__INCLUDED_)
#define AFX_TCPPORTSCANNER_H__9827BC61_F74C_408D_8D12_A99E0FBC4A9F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma warning(disable:4786)

#include "TCPSocketAsync.h"

#include <deque>
#include <map>
#include <set>

KOMODIA_NAMESPACE_START

class CGenericCriticalSection;

class CTCPPortScanner : public CErrorHandler
{
public:
	//Our list of active ports
	typedef std::set<unsigned short> PortsList;
public:
	//Set/get max sockets
	void SetMaxSockets(int iMaxSockets);
	int GetMaxSockets()const;

	//Set/get the connection timeout
	void SetConnectionTimeout(int iMS);
	int GetConnectionTimeout()const;

	//Get the data for the port
	const char* GetData(unsigned short usPort)const;

	//Get the data for the port
	char* GetDataForModify(unsigned short usPort)const;

	//Get the ports list
	const PortsList& GetPortList()const;

	//Status of the scanner
	BOOL IsDone()const;
	BOOL IsError()const;

	//Set/get we can receive data (0 for none)
	void SetReceiveData(int iMaxDataSize,
						DWORD dwTimeout);
	DWORD GetReceiveDataTimeout()const;
	int GetReceiveDataSize()const;

	//Do the scan
	virtual BOOL Scan(const std::string& rDestinationAddress)=0;
	virtual BOOL Scan(IP aTarget)=0;

	//Stop the scan
	virtual BOOL StopScan();

	//Add ports to scan
	void AddPorts(unsigned short usFromPort,
				  unsigned short usToPort);

	//Ctor and Dtor
	CTCPPortScanner();
	virtual ~CTCPPortScanner();
protected:
	//Events for user to override
	//Port status
	//usPort - The port
	//bConnected - Did we manager to connect?
	virtual void OnPort(unsigned short usPort,
						BOOL bConnected);

	//Port status with data (port sent data)
	//usPort - The port
	//pData - The data
	//iDataSize - The data size
	virtual void OnPortData(unsigned short usPort,
							const char* pData,
							int iDataSize);

	//Scan is done (for user to override)
	virtual void TCPScanDone(BOOL bError);

	//Scan is done (for library)
	virtual void TCPScanDoneLib(BOOL bError);

	//Port was scanned (no resolution yet, for lib to overide)
	virtual void PortScanned(unsigned short usPort);

	//We are done
	void ScanDone(BOOL bError,
				  BOOL bSpawnThread=TRUE);

	//Do we have something to scan
	BOOL CanScan();

	//Adjust the number of sockets
	int AdjustNumberOfSockets(int iIncrement);

	//Get the next port
	unsigned short GetNextPort();

	//When socket is done
	void SocketDone(unsigned short usPort,
					BOOL bConnected);

	//Socket done and received data
	void SocketDone(unsigned short usPort,
					char* pData,
					int iDataSize);

	//Delete the ports map
	void DeletePortsMap();

	//Actual a new socket
	//Derived class should override this for their sockets
	virtual CTCPSocketAsync* AllocateSocket(unsigned short usPort)const;

	//Unallocate the socket
	virtual void DestroySocket(CTCPSocketAsync* pSocket)const;

	//Create a new socket
	BOOL NewSocket();
	
	//Set there's an error
	void SetError(BOOL bError);

	//Set the done flag
	void SetDone(BOOL bDone);

	//Reset the socket count
	void ResetSocketCount();

	//Reset pair data
	void ResetPair();

	//Reset the data
	void ResetData();

	//Set/get the target address
	void SetTarget(IP aTarget);
	IP GetTarget()const;

	//Are we finished (before done, for internal uses)
	BOOL IsFinished()const;
private:
	//Our TCP socket
	class CTCPScanner : public CTCPSocketAsync
	{
	public:
		//Ctor and Dtor
		CTCPScanner(CTCPPortScanner* pFather,
					unsigned short usPort,
					int iDataSize,
					DWORD dwTimeout);
		virtual ~CTCPScanner();
	protected:
		//Our events
		NO_OnSocketAccept
		NO_OnSocketOOB
		NO_OnSocketWrite

		virtual BOOL OnSocketTimeout();
		virtual BOOL OnSocketConnect(int iErrorCode);
		virtual BOOL OnSocketClose(int iErrorCode);
		virtual BOOL OnSocketReceive(int iErrorCode);
	private:
		//Our father
		CTCPPortScanner* m_pFather;

		//Do we receive data
		int m_iDataSize;

		//Do we have a timeout
		DWORD m_dwTimeout;

		//The port
		unsigned short m_usPort;
	};

	friend CTCPScanner;
private:
	//Structure of ports to scan
	typedef struct _ScanPair
	{
		unsigned short usFromPort;
		unsigned short usToPort;
	} ScanPair;

	//Our list of ports to scan
	typedef std::deque<ScanPair> PortsVector;

	//Our map of data
	typedef std::map<unsigned short,char*> DataMap;

	//Thread data
	typedef struct _ThreadData
	{
		CTCPPortScanner*	pScanner;
		BOOL				bError;
	} ThreadData;
private:
	//No copy Ctor
	CTCPPortScanner(const CTCPPortScanner& rScanner);

	//No copy operator
	CTCPPortScanner& operator=(const CTCPPortScanner& rScanner);

	//Our thread
	static DWORD DoneThread(LPVOID lpData);

	//Are we done ?
	BOOL m_bDone;

	//Our CS
	CGenericCriticalSection* m_pCSection;

	//Done CS
	CGenericCriticalSection* m_pCSectionDone;

	//Our answered ports list
	PortsList m_aPortList;

	//List of all answers
	PortsList m_aPortAnswers;

	//Our ports to scan
	PortsVector m_aPortsToScan;

	//Do we receive data
	int m_iReceiveData;

	//Timeout
	DWORD m_dwTimeout;

	//Number of max sockets
	int m_iMaxSockets;

	//Current pair we are working on
	ScanPair m_aCurrentPair;

	//Did we have an error
	BOOL m_bError;

	//The ports data map
	DataMap m_aPortsData;

	//Number of active sockets
	int m_iActiveSockets;

	//Our target
	IP m_aTarget;

	//Connection timeout
	int m_iConnectionTimeout;

	//Did we finish (not for user indication)
	BOOL m_bFinished;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TCPPORTSCANNER_H__9827BC61_F74C_408D_8D12_A99E0FBC4A9F__INCLUDED_)
