/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TRACEROUTESOCKETREVERSEDNS_H__55BEFFAA_FEFF_483E_90A7_B2F1D3A7A13F__INCLUDED_)
#define AFX_TRACEROUTESOCKETREVERSEDNS_H__55BEFFAA_FEFF_483E_90A7_B2F1D3A7A13F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TraceRouteSocket.h"

namespace KomodiaDNS
{
	class CDNSManager;
}

KOMODIA_NAMESPACE_START

class CGenericThread;

class CTraceRouteSocketReverseDNS : public CTraceRouteSocket  
{
public:
	//Our data
	typedef struct _DNSTraceRouteData : public CTraceRouteSocket::_TraceRouteData
	{
		std::string		sReverseDNS;
	} DNSTraceRouteData;

	//The data we pass back
	typedef std::vector<DNSTraceRouteData> DNSTraceRouteVector;
public:
	//Are we initialized
	BOOL IsInitialized()const;

	//Do the trace
	virtual BOOL TraceRoute(IP aDestinationAddress,
							DWORD dwTimeout,
							unsigned char ucTTL=30);

	//Take the data
	void TakeDNSData(DNSTraceRouteVector& rVector);

	//Get the data
	DNSTraceRouteVector& GetDNSData();

	//Is the traced done
	virtual BOOL IsTraceDone()const;

	//Ctor and Dtor
	CTraceRouteSocketReverseDNS(const std::string& rDNSServer,
								BOOL bDNSTcp);
	virtual ~CTraceRouteSocketReverseDNS();
protected:
	//ctor
	CTraceRouteSocketReverseDNS(std::string sDNSServer,
								BOOL bDNSTcp,
								BOOL bAsync);

	//Is the trace done
	virtual void DNSTraceDone();
private:
	//No copy Ctor
	CTraceRouteSocketReverseDNS(const CTraceRouteSocketReverseDNS& rTrace);

	//No copy operator
	CTraceRouteSocketReverseDNS& operator=(const CTraceRouteSocketReverseDNS& rTrace);

	//Event for user to overide
	virtual void TraceDone();

	//For user to overide, per TTL data
	virtual void TraceData(const TraceRouteData& rData);

	//Thread for DNS
	static DWORD DNSResolving(LPVOID lpData);

	//Our data
	DNSTraceRouteVector m_aData;

	//Are we async
	BOOL m_bAsync;

	//Our thread
	CGenericThread* m_pThread;

	//Our DNS manager
	KomodiaDNS::CDNSManager* m_pDNSManager;

	//Is the trace done ?
	BOOL m_bTraceDone;

	//Are we initialized ?
	BOOL m_bInitialized;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TRACEROUTESOCKETREVERSEDNS_H__55BEFFAA_FEFF_483E_90A7_B2F1D3A7A13F__INCLUDED_)
