/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "TraceLog.h"

#include <atlbase.h>

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

KOMODIA_NAMESPACE_START

CTraceLog::CTraceLog() : CErrorHandler::CErrorLog()
{
}

CTraceLog::~CTraceLog()
{
}

void CTraceLog::WriteLog(const std::string& rClass, 
						 const std::string& rMethod, 
						 const std::string& rMessage, 
						 CErrorHandler::LogPriority aPriority)
{
	try
	{
		//Our error string
		std::string sPrefix;

		//Our string to print
		switch (aPriority)
		{
		case CErrorHandler::lpMessage:
			sPrefix="***MESSAGE*** ";
			break;
		case CErrorHandler::lpCritical:
			sPrefix="***CRITICAL*** ";
			break;
		case CErrorHandler::lpError:
			sPrefix="***ERROR*** ";
			break;
		}

		//Write to log
		ATLTRACE("%s %s, %s, %s\n",sPrefix.c_str(),
								   rClass.c_str(),
								   rMethod.c_str(),
								   rMessage.c_str());
	}
	catch (...)
	{
	}
}

void CTraceLog::ReportError(const std::string& rClass,
						    const std::string& rMethod,
						    const std::string& rMessage)
{
	//Delegate call
	WriteLog(rClass,
			 rMethod,
			 rMessage,
			 CErrorHandler::lpError);
}

void CTraceLog::WriteMessage(const std::string& rClass,
							 const std::string& rMethod,
							 const std::string& rMessage,
							 CErrorHandler::LogPriority aPriority)
{
	//Delegate call
	WriteLog(rClass,
			 rMethod,
			 rMessage,
			 aPriority);
}

KOMODIA_NAMESPACE_END
