/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#include "stdafx.h"
#include "UDPCrafter.h"

#include "ErrorHandlerMacros.h"
#include "Array_ptr.h"

#include <memory>

#ifdef _MEMORY_DEBUG 
	#define new	   DEBUG_NEW  
	#define malloc DEBUG_MALLOC  
    static char THIS_FILE[] = __FILE__;  
#endif

	KOMODIA_NAMESPACE_START

#define CUDPCrafter_Class "CUDPCrafter"

CUDPCrafter::CUDPCrafter() : CUDPSocket(TRUE),
							 CIPCrafter()
{
	try
	{
		//Set our name
		CUDPSocket::SetName(CUDPCrafter_Class);

		//Set the default
		SetUDPDefault();

		//Set the protocol
		SetProtocol(IPPROTO_UDP);
	}
	ERROR_HANDLER("CUDPCrafter")
}

CUDPCrafter::~CUDPCrafter()
{
}

LPIpHeader CUDPCrafter::ConstructIPHeader(unsigned char  ucProtocol,
										  unsigned short usFragmentationFlags,
										  unsigned char  ucTTL,
										  unsigned short usIdentification,
										  unsigned char  ucHeaderLength)const
{
	return ConstructCraftedIPHeader(ucProtocol,
									usFragmentationFlags,
									ucTTL,
									usIdentification,
									ucHeaderLength);
}

void CUDPCrafter::FinalIPHeader(LPIpHeader lpHead)const
{
	try
	{
		//Check if we need to modify the header
		if (!m_DefaultChecksum)
			lpHead->usChecksum=htons(m_usChecksum);
	}
	ERROR_HANDLER("FinalIPHeader")
}

void CUDPCrafter::SetUDPDefault()
{
	try
	{
		//Default checksum used
		m_UDPDefaultChecksum=TRUE;
	}
	ERROR_HANDLER("SetUDPDefault")
}

void CUDPCrafter::SetUDPChecksum(unsigned short usChecksum)
{
	m_usUDPChecksum=usChecksum;
}

void CUDPCrafter::SetUDPUseDefaultChecksum(BOOL bDefault)
{
	m_UDPDefaultChecksum=bDefault;
}

void CUDPCrafter::FinalUDPHeader(LPUDPHeader lpHeader)const
{
	if (!m_UDPDefaultChecksum)
		lpHeader->usChecksum=htons(m_usUDPChecksum);
}

char* CUDPCrafter::GetCraftedPacket(unsigned short usSourcePort,
								    unsigned long ulDestinationAddress,
								    const char* pBuffer,
								    unsigned long ulBufferSizeBufferSize,
								    unsigned short usDestinationPort,
								    unsigned short& usTotalSize)const
{
	try
	{
		//Create the header
		UDPHeader aHeader;

		//Set the ports
		aHeader.usSourcePort=htons(usSourcePort);
		aHeader.usDestinationPort=htons(usDestinationPort);

		//Check sum
		aHeader.usChecksum=0;

		//Set the total size
		unsigned long ulTotalLength;
		ulTotalLength=UDPHeaderLength+ulBufferSizeBufferSize;

		//Set the length
		aHeader.usLength=htons(ulTotalLength);

		//Allocate the buffer
		char* pNewBuffer;
		pNewBuffer=new char[ulTotalLength];

		//Protect the data
		CArray_ptr<char> pProtection(pNewBuffer);

		//Copy original header
		memcpy(pNewBuffer,
			   &aHeader,
			   UDPHeaderLength);

		//Do we have data ?
		if (ulBufferSizeBufferSize)
			memcpy(pNewBuffer+UDPHeaderLength,
				   pBuffer,
				   ulBufferSizeBufferSize);

		//Get the checksum
		aHeader.usChecksum=CalculatePseudoChecksum(pNewBuffer,
												   ulTotalLength,
												   ulDestinationAddress,
												   ulTotalLength);

		//Recopy it
		memcpy(pNewBuffer,
			   &aHeader,
			   UDPHeaderLength);

		//Set the new checksum (if applicateable)
		FinalUDPHeader((LPUDPHeader)pNewBuffer);

		//And return what we have from the IP
		return CIPCrafter::GetCraftedPacket(this,
											ulDestinationAddress,
											pNewBuffer,
											ulTotalLength,
											usTotalSize);
	}
	ERROR_HANDLER_RETURN("GetCraftedPacket",NULL)
}

KOMODIA_NAMESPACE_END